from conans import ConanFile, CMake, tools


class CaosdbConan(ConanFile):
    name = "caosdb"
    version = "0.2.2"
    license = "AGPL-3.0-or-later"
    author = "Timm C. Fitschen <t.fitschen@indiscale.com>"
    url = "https://gitlab.indiscale.com/caosdb/src/caosdb-cpplib.git"
    description = "C++ library for the CaosDB project"
    topics = ("data management", "caosdb")
    settings = "os", "compiler", "build_type", "arch"
    options = {
        "shared": [True, False],
        "fPIC": [True, False],
        "build_acm": [True, False],
    }
    default_options = {
        "shared": False,
        "fPIC": True,
        "build_acm": False,
    }
    generators = "cmake"
    requires = [
        ("grpc/1.48.0"),
    ]
    build_requires = [
        ("boost/1.78.0"),
        ("gtest/1.11.0"),
    ]
    exports = ("*.cmake", "*CMakeLists.txt", "*.in",
               "*.h", "*.proto", "*.c", "*.cpp",
               "*.rst", "*.md",
              )
    exports_sources = "src", "doc", "include", "test", "cmake", "proto"

    def config_options(self):
        if self.settings.os == "Windows":
            del self.options.fPIC
        self.options["boost"].without_python = True
        self.options["boost"].filesystem_version = "3"

    # def source(self):
        # self.run("git clone https://gitlab.indiscale.com/caosdb/src/caosdb-cpplib.git")
        # self.run("cd caosdb-cpplib && git submodule update --init")

    def imports(self):
        self.copy("protoc*", "build_tools", "bin")
        self.copy("grpc_cpp_plugin*", "build_tools", "bin")

    def build(self):
        cmake = CMake(self)
        if self.options.build_acm:
            cmake.definitions["BUILD_ACM"] = "On"

        cmake.configure(source_folder="")
        cmake.build()

    def package(self):
        self.copy("*.h", dst="include", src="include")
        self.copy("*.dll", dst="bin", keep_path=False)
        self.copy("*.so", dst="lib", keep_path=False)
        self.copy("*.dylib", dst="lib", keep_path=False)
        self.copy("*.a", dst="lib", keep_path=False)

    def package_info(self):
        self.cpp_info.libs = ["caosdb", "ccaosdb"]
